/* 
   HCIDump - HCI packet analyzer	
   Copyright (C) 2000-2001 Maxim Krasnyansky <maxk@qualcomm.com>
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as
   published by the Free Software Foundation;
   
   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
   IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY CLAIM,
   OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER
   RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
   NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
   USE OR PERFORMANCE OF THIS SOFTWARE.
   
   ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, COPYRIGHTS,
   TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS SOFTWARE IS DISCLAIMED.
*/

/*
 * Id: parser.h,v 1.21 2003/09/07 16:21:48 holtmann Exp
 * $Id: parser.h,v 1.4 2003/09/12 23:38:11 max Exp $
 */

struct frame {
	void	*data;
	int	data_len;
	void	*ptr;
	int	len;
	int 	in;
	int	handle;
	long	flags;
	struct	timeval ts;
};

/* Parser flags */
#define DUMP_WIDTH	20

#define DUMP_HEX	0x01
#define DUMP_ASCII	0x02
#define DUMP_TYPE_MASK	(DUMP_HEX | DUMP_ASCII)
#define DUMP_TSTAMP	0x04
#define DUMP_RAW	0x08

/* Parser filter */
#define FILT_HCI	0x0001
#define FILT_L2CAP	0x0002
#define FILT_RFCOMM	0x0004
#define FILT_SDP	0x0008
#define FILT_SCO	0x0010
#define FILT_BNEP	0x0020
#define FILT_CMTP	0x0040
#define FILT_HIDP	0x0080

#define STRUCT_OFFSET(type, member)  ((uint8_t *)&(((type *)NULL)->member) - \
                                     (uint8_t *)((type *)NULL))

#define STRUCT_END(type, member)     (STRUCT_OFFSET(type, member) + \
                                     sizeof(((type *)NULL)->member))

#define p_filter(f)	(!(parser.filter & (f)))

/* XXX use memmove so gcc will not use builtin memcpy */
#define get_unaligned(ptr) \
({ __typeof__ (*(ptr)) __tmp; memmove(&__tmp, (ptr), sizeof(__tmp)); __tmp; })

struct parser_t {
	unsigned long flags;
	unsigned long filter;
	unsigned int defpsm;
	int state;
};

extern struct parser_t parser;

void init_parser(unsigned long flags, unsigned long filter, 
	unsigned int assume_psm);

void p_indent(int level, struct frame *f);

/* get_uXX functions do byte swaping */

uint8_t get_u8(struct frame *frm);
uint16_t get_u16(struct frame *frm);
uint32_t get_u32(struct frame *frm);
uint64_t get_u64(struct frame *frm);
void get_u128(struct frame *frm, uint64_t *l, uint64_t *h);

char *get_uuid_name(int uuid);

void raw_dump(int level, struct frame *frm);
void raw_ndump(int level, struct frame *frm, int num);
void hci_dump(int level, struct frame *frm);
void l2cap_dump(int level, struct frame *frm);
void rfcomm_dump(int level, struct frame *frm);
void sdp_dump(int level, struct frame *frm);
void bnep_dump(int level, struct frame *frm);
void cmtp_dump(int level, struct frame *frm);
void hidp_dump(int level, struct frame *frm);

void parse(struct frame *frm);
